package org.netkernel.cp;

import java.util.*;

import org.netkernel.container.ISpaceListener;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.netkernel.xml.xda.IXDAReadOnly;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class WrapperAccessor extends StandardAccessorImpl  implements ISpaceListener
{
	public static final String HEADER_CP="WrappedControlPanel";
	public static final String HEADER_CP_SUBTITLE="WrappedControlPanelSubtitle";
	public static final String HEADER_CP_ICON="WrappedControlPanelIcon";
	public static final String HEADER_CP_MIME="WrappedControlPanelMime";
	private static final String GATEKEEPER_SERVICE="cp:gatekeeper";
	private Boolean mGateKeeperStatus;
	
	private static final Set sGKExclusions = new HashSet();
	static
	{	sGKExclusions.add("cp:authdisplay");
		sGKExclusions.add(GATEKEEPER_SERVICE);
		sGKExclusions.add("ControlPanelDiscovery");
	}
	
	public WrapperAccessor()
	{	this.declareThreadSafe();		
	}
		
	@Override
	protected void postCommission(INKFRequestContext aContext) throws Exception
	{	super.postCommission(aContext);
		aContext.getKernelContext().getKernel().addSpaceListener(this);
	}

	@Override
	protected void preDecommission(INKFRequestContext aContext) throws Exception
	{	aContext.getKernelContext().getKernel().removeSpaceListener(this);
		super.preDecommission(aContext);
	}

	private String securityCheck(INKFRequestContext aContext, INKFRequestReadOnly aInitialRequest) throws Exception
	{	
		
		if (mGateKeeperStatus==null)
		{	try
			{	aContext.meta(GATEKEEPER_SERVICE);
				mGateKeeperStatus=Boolean.TRUE;
			}
			catch (NKFException e)
			{	mGateKeeperStatus=Boolean.FALSE;
			}
		}
		String result=null;
		if (mGateKeeperStatus)
		{	String id=aInitialRequest.getResolvedElementId();
			if (!sGKExclusions.contains(id))
			{	INKFRequest req=aContext.createRequestToEndpoint(GATEKEEPER_SERVICE);
				req.addArgument("endpointId", id);
				req.setRepresentationClass(String.class);
				result = (String)aContext.issueRequest(req);
			}
		}
		return result;
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String type=aContext.getThisRequest().getArgumentValue(INKFRequestReadOnly.ARG_ACTIVE_TYPE);
		if (type.equals("controlPanelWrapperPre"))
		{	pre(aContext);
		}
		else
		{	post(aContext);
		}
	}
	
	public void pre(INKFRequestContext aContext) throws Exception
	{	
		INKFRequestReadOnly initialRequest = aContext.source("arg:request", INKFRequestReadOnly.class);
		String redirect = securityCheck(aContext,initialRequest);
		if (redirect==null)
		{	INKFRequest innerRequest = initialRequest.getIssuableClone();
			innerRequest.setRepresentationClass(Object.class);
			aContext.createResponseFrom(innerRequest);
		}
		else
		{	INKFRequest redirectReq = aContext.createRequest(redirect);
			redirectReq.addArgumentByValue("request", initialRequest);
			aContext.createResponseFrom(redirectReq);
		}
	}
	
	public void post(INKFRequestContext aContext) throws Exception
	{
		INKFResponseReadOnly respIn=aContext.sourceForResponse("arg:response");
		if (respIn.hasHeader(HEADER_CP))
		{
			String setTitle=null;
			if (respIn.getHeader(HEADER_CP) instanceof String)
			{	setTitle=(String)respIn.getHeader(HEADER_CP);
			}
			
			String title;
			String subtitle;
			String icon;

			INKFRequestReadOnly reqIn=aContext.source("arg:request",INKFRequestReadOnly.class);
			String path=reqIn.getIdentifier();
			int i=path.indexOf(":");
			path=path.substring(i+1);
			
			//skip past the value space (containing req/resp) to ensure cachability
			IRequestScopeLevel configScope=aContext.getKernelContext().getRequestScope();
			configScope=configScope.getParent().getParent().getParent().getParent();
			INKFRequest configReq = aContext.createRequest("active:controlPanelConfig");
			configReq.setRepresentationClass(ControlPanelConfig.class);
			configReq.setRequestScope(configScope);
			ControlPanelConfig config=(ControlPanelConfig)aContext.issueRequest(configReq);
			
			IHDSNode iconNode=config.getIconForPath(path);
			if (iconNode!=null)
			{
				title=iconNode.getFirstValue("title").toString();
				Object subtitleValue=iconNode.getFirstValue("subtitle");
				if (subtitleValue!=null)
				{	subtitle=subtitleValue.toString();
				}
				else
				{	subtitle="";
				}
				Object iconValue=iconNode.getFirstValue("icon");
				if (iconValue!=null)
				{	icon=iconValue.toString();
				}
				else
				{	icon="/nkse/style/img/noicon.png";
				}
			}
			else
			{	title=path;
				subtitle="";
				icon="/nkse/style/img/noicon.png";
			}
			
			if (setTitle!=null)
			{	title=setTitle;
			}
			
			if (respIn.hasHeader(HEADER_CP_SUBTITLE) && respIn.getHeader(HEADER_CP_SUBTITLE) instanceof String)
			{	subtitle=(String)respIn.getHeader(HEADER_CP_SUBTITLE);
			}
			if (respIn.hasHeader(HEADER_CP_ICON) && respIn.getHeader(HEADER_CP_ICON) instanceof String)
			{	icon=(String)respIn.getHeader(HEADER_CP_ICON);
			}
			
			String mime;
			if (respIn.hasHeader(HEADER_CP_MIME) && respIn.getHeader(HEADER_CP_MIME) instanceof String)
			{	mime=(String)respIn.getHeader(HEADER_CP_MIME);
			}
			else
			{	mime="text/html";
			}
			
			Object contentRep=convertButtons(respIn, aContext);
			
			INKFRequest req1=aContext.createRequest("active:xslt");
			req1.addArgument("operator","res:/nkse/style/xsl/styleCPTabs.xsl");
			req1.addArgumentByValue("operand",config.getPanels());
			if(iconNode!=null)
			{	String panelId=(String)iconNode.getFirstValue("panel");
				req1.addArgumentByValue("panelId",panelId);
			}
			req1.setRepresentationClass(Document.class);
			INKFResponseReadOnly tabs=aContext.issueRequestForResponse(req1);
			
			INKFRequest req=aContext.createRequest("active:xrl2");
			req.addArgument("template", "res:/nkse/style/template/cp-template.xml");
			req.addArgumentByValue("pagetitle", "NetKernel - "+title);
			req.addArgumentByValue("tooltitle", title);
			req.addArgumentByValue("subtitle", subtitle);
			req.addArgumentByValue("icon", icon);
			req.addArgumentFromResponse("tabs",tabs);
			req.addArgumentByValue("content",contentRep);
			req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
			INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
			INKFResponse respOut=aContext.createResponseFrom(resp);
			respOut.setMimeType(mime);
		}
		else
		{	INKFResponse response=aContext.createResponseFrom(respIn);
		}
	}
	
	static Object convertButtons(INKFResponseReadOnly aRespIn, INKFRequestContext aContext) throws Exception
	{
		Object contentRep=aRespIn.getRepresentation();
		Document content;
		if (contentRep instanceof Document)
		{	content=(Document)contentRep;
		}
		else
		{	content=aContext.transrept(contentRep, Document.class);
			contentRep=content;
		}
		
		Node root=content.getDocumentElement();
		Element e=content.getDocumentElement();
		while ((e=XMLUtils.inOrderTraversalNext(e,root))!=null)
		{	
			String classAtt=e.getAttribute("class");
			if (classAtt.equals("btn"))
			{	String name=e.getNodeName().toLowerCase();
				Node tt=null;
				boolean submit=false;
				String onclick=null;
				if (name.equals("input"))
				{	
					if ( e.getAttribute("type").equals("submit") )
	                {	submit=true;
	                }
	                if (e.getAttribute("onclick").length()>0)
	                {	onclick=e.getAttribute("onclick");
	                }					
					
					tt=content.createTextNode(e.getAttribute("value"));
					Element a1=content.createElement("a");
					a1.setAttribute("class", classAtt+"2");
					String id=e.getAttribute("id");
					if (id.length()>0)
					{	a1.setAttribute("id", id);
					}
					e.getParentNode().replaceChild(a1, e);
					e=a1;
				}
				else if (name.equals("a"))
				{	tt = e.getFirstChild();
					e.setAttribute("class", classAtt+"2");
				}
				
				if (tt!=null)
				{	
					Element  i1 = content.createElement("i"); i1.appendChild(content.createTextNode(" "));
					Element  i2 = content.createElement("i"); i2.appendChild(content.createTextNode(" "));
					Element  s1 = content.createElement("span");
					Element  s2 = content.createElement("span"); s2.appendChild(content.createTextNode(" "));
		            e.appendChild(s1);
		            s1.appendChild(i1);
		            s1.appendChild(s2);
		            s1.appendChild(tt);
		            e.insertBefore(i2,s1);
		            
		            if (submit)
			        {	e.setAttribute("onclick", "submitForm(this)");
			        }
			        if (onclick!=null)
			        {	e.setAttribute("onclick", onclick);
			        }
				}
				
				
			}
		}
			
		return contentRep;
	}

	public void spaceChanged()
	{	//reset gatekeeper status to see if service has appeared or disappeared
		mGateKeeperStatus=null;
	}
	
	
}
